!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! LSmove
!
!   Least Squares fit --
!   Estimates equilibrium values of generalized positions given          
!     Niter values of E, generalized forces, and generalized positions
!
!   Edata is an array of the total energies (or negative of cohesive E)
!      for each iteration
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
Subroutine LSmove(Niter,Nfreedom,Edata,Fdata,Rdata,Efit,Rfit,&
           ForceTol,Maxmove,ConditionNo,Success)

 Integer, Intent(IN) :: Niter, Nfreedom
 Real,    Intent(IN) :: ForceTol,Maxmove,ConditionNo
 Real,Intent(INOUT)  :: Rdata(Nfreedom,Niter),Fdata(Nfreedom,Niter),Edata(Niter)
 Real,    Intent(OUT):: Efit,Rfit(Nfreedom)
 Logical, Intent(OUT):: Success

  Real, allocatable  :: A(:,:),U(:,:),VT(:,:),S(:),Work(:),Rref(:),B(:)
  Integer, allocatable :: IWork(:),Index(:)
  Integer :: N, M, Lwork, Liwork, Info, i,j,k,icount ,p ,pp
  Real :: x,weight,Emin

  Success = .false.
  Efit=0;Rfit=0

  If (Niter.lt.2) Return

  M=Niter
  N=Nfreedom+1

  Lwork=8*N*N+12*N
  Liwork=8*N

  Allocate(A(N,N),S(N),U(N,N),VT(N,N),Work(Lwork),IWork(Liwork),&
           Rref(Nfreedom),B(N),Index(Niter),stat=info)
    If (info /= 0) then
       write(Error_Unit,*) 'Error in Fitmove allocation ', info
       call flush_(Error_Unit)
       stop
    Endif

  Call Insertion_Sort(Edata(1:Niter),Index,.TRUE.)  !Order in increasing E

  ! Use lowest energy iteration as reference

  Rref(1:Nfreedom)=Rdata(1:Nfreedom,Index(1))
  Emin = Edata(Index(1))
  Edata(1:Niter) = Edata(1:Niter)-Emin

  A=0 ; B=0 ; S=0
  A(1:N,1)=1

  Do i=1,M
       j=Index(i)
       Rdata(1:Nfreedom,j)=Rdata(1:Nfreedom,j)-Rref(1:Nfreedom)
       weight= 1.0/(((Edata(j)-Emin)/ForceTol)**2 + 1)
       S(1)=S(1)+weight
       B(1)=B(1)+weight*Edata(j)

        Do p=1,Nfreedom
           x=Fdata(p,j)*weight
           S(p+1)=S(p+1) + x
           A(1,p+1)=A(1,p+1) + x
           B(1)=B(1) + 0.5*x*Rdata(j)
           B(p+1)=B(p+1) + x*Edata(j)

           Do pp=1,Nfreedom
              A(p+1,pp+1)=A(p+1,pp+1) + x*Fdata(pp,j)
              B(p+1)=B(p+1) + 0.5*x*Fdata(pp,j)*Rdata(pp,j)
           EndDo
        EndDo

  EndDo

        B(1)=B(1)/S(1)
        Do p=1,Nfreedom
           A(1,p+1)=A(1,p+1)/S(1)
           B(p+1)=B(p+1)/S(p+1)
           A(p+1,2:N)=A(p+1,2:N)/S(p+1)
        EndDo

  Call DGESDD('A',M,N,A,M,S,U,M,VT,N,Work,Lwork,Iwork,info)
     Write(Log_Unit,*) 'completed DGESDD with info = ',info
     Write(Log_Unit,'(1p5e15.7)') ' singular values ', (S(i),i=1,icount)
     If (info /= 0) then
       write(Error_Unit,*) 'Error in Fitmove RDGESDD ', info
       call flush_(Error_Unit)
       stop
     Endif
  
  Work=0
  x=ABS(S(1))/ConditionNo
  Do i=1,icount
     If (ABS(S(i)).gt.x) then
        Work(1:N)=Work(1:N)+VT(i,1:N)*DOT_PRODUCT(U(1:N,i),B)/S(i)
     EndIf
  Enddo

  Efit=Work(1)
  Rfit(1:Nfreedom)=Rref(1:Nfreedom)+Work(2:N)      

   write(Log_Unit,*) ' Efit = ',Efit

   write(Log_Unit,*) ' Rfit = ',Rfit

 ! Check MaxMove:
      x=-100
      Do i=1,Nfreedom
         x=Max(x,ABS(Rref(i)-Rfit(i)))
      Enddo

    Write(Log_Unit,*) 'error measure = ',x

   if (x <= MaxMove) Success=.true.
   Deallocate(A,S,U,VT,Work,IWork,Rref,B,Index)
 
   Return

End subroutine LSMove



